#include <stdio.h>
#include <memory.h>
#include "mz2000.h"
#include <gdk/gdkx.h>
#include <X11/Xlib.h>

static struct keys {
	char *KeyName;
	u8 MZKeyVal;
} keysyms[] = {
	{"F1", 0}, /* F1 */
	{"F2", 1}, /* F2 */
	{"F3", 2}, /* F3 */
	{"F4", 3}, /* F4 */
	{"F5", 4}, /* F5 */
	{"F6", 5}, /* F6 */
	{"F7", 6}, /* F7 */
	{"F8", 7}, /* F8 */

	{"F9", 8}, /* F9 */
	{"F10", 9}, /* F10 */
	{"KP_8", 10}, /* 8 (tenkey) */
	{"KP_9", 11}, /* 9 (tenkey) */
	{"", 12}, /* 00 (tenkey) */
	{"KP_Decimal", 13}, /* . (tenkey) */
	{"KP_Add", 14}, /* + (tenkey) */
	{"KP_Subtract", 15}, /* - (tenkey) */

	{"KP_0", 16}, /* 0 (tenkey) */
	{"KP_1", 17}, /* 1 (tenkey) */
	{"KP_2", 18}, /* 2 (tenkey) */
	{"KP_3", 19}, /* 3 (tenkey) */
	{"KP_4", 20}, /* 4 (tenkey) */
	{"KP_5", 21}, /* 5 (tenkey) */
	{"KP_6", 22}, /* 6 (tenkey) */
	{"KP_7", 23}, /* 7 (tenkey) */

	{"Tab", 24}, /* TAB */
	{"space", 25}, /* SPACE */
	{"Return", 26}, /* CR */
	{"Up", 27}, /* UP */
	{"Down", 28}, /* DOWN */
	{"Left", 29}, /* LEFT */
	{"Right", 30}, /* RIGHT */
	{"Break", 31}, /* BREAK */
	{"Pause", 31}, /* BREAK */
 
	{"slash", 32}, /* / */
	{"A", 33}, /* A */
	{"B", 34}, /* B */
	{"C", 35}, /* C */
	{"D", 36}, /* D */
	{"E", 37}, /* E */
	{"F", 38}, /* F */
	{"G", 39}, /* G */

	{"H", 40}, /* H */
	{"I", 41}, /* I */
	{"J", 42}, /* J */
	{"K", 43}, /* K */
	{"L", 44}, /* L */
	{"M", 45}, /* M */
	{"N", 46}, /* N */
	{"O", 47}, /* O */
 
	{"P", 48}, /* P */
	{"Q", 49}, /* Q */
	{"R", 50}, /* R */
	{"S", 51}, /* S */
	{"T", 52}, /* T */
	{"U", 53}, /* U */
	{"V", 54}, /* V */
	{"W", 55}, /* W */

	{"X", 56}, /* X */
	{"Y", 57}, /* Y */
	{"Z", 58}, /* Z */
	{"asciicircum", 59}, /* ^ */
	{"backslash", 60}, /* \ */
	{"question", 61}, /* ? */
	{"period", 62}, /* . */
	{"comma", 63}, /* , */

	{"0", 64}, /* 0 */
	{"1", 65}, /* 1 */
	{"2", 66}, /* 2 */
	{"3", 67}, /* 3 */
	{"4", 68}, /* 4 */
	{"5", 69}, /* 5 */
	{"6", 70}, /* 6 */
	{"7", 71}, /* 7 */

	{"8", 72}, /* 8 */
	{"9", 73}, /* 9 */
	{"colon", 74}, /* : */
	{"semicolon", 75}, /* ; */
	{"minus", 76}, /* - */
	{"at", 77}, /* @ */
	{"bracketleft", 78}, /* [ */
	{"", 79},

	{"bracketright", 80}, /* ] */
	{"", 81},
	{"Home", 82}, /* CLR/HOME */
	{"Insert", 83}, /* INST/DEL */
	{"Delete", 83}, /* INST/DEL */
	{"BackSpace", 83}, /* INST/DEL */
	{"", 84},
	{"", 85},
	{"", 86},
	{"", 87},

	{"Alt_L", 88}, /* GRPH */
	{"Caps_Lock", 89}, /* SFT LOCK */
	{"Shift_L", 90}, /* SHIFT */
	{"Shift_R", 90}, /* SHIFT */
	{"Escape", 91}, /* KANA */
	{"Hankaku", 91}, /* KANA */
	{"Katakana", 91}, /* KANA */
	{"", 92},
	{"", 93},
	{"", 94},
	{"", 95},
	{NULL, 255},
};


int init_keyboard(MZ2000 *mz2000)
{
	int i;
	struct keys *curp = keysyms;

	if ((mz2000->keyboard = (MZKeyboard *)malloc(sizeof(MZKeyboard))) == NULL) {
		fprintf(stderr, "No enough memory\n");
		return 0;
	}

	for (i = 0; i < 256; i++) {
		mz2000->keyboard->kcode[i] = 0xff;
	}
	while (curp->KeyName != NULL) {
		if (strcmp(curp->KeyName, "")) {
			KeyCode keycode;
			guint keysym;
			keysym = gdk_keyval_from_name(curp->KeyName);
			keycode = XKeysymToKeycode(GDK_DISPLAY(), keysym);
			mz2000->keyboard->kcode[keycode] = curp->MZKeyVal;
		}
		curp++;
	}
	gdk_key_repeat_disable();
	pthread_mutex_init(&(mz2000->keyboard_mutex), NULL);
	keyboard_releaseall(mz2000);
	return 1;
}

int keyboard_getbitdata(MZ2000 *mz2000, int strobe)
{
	int retval;
	pthread_mutex_lock(&(mz2000->keyboard_mutex));
	retval = mz2000->keyboard->key_bitdata[strobe];
	pthread_mutex_unlock(&(mz2000->keyboard_mutex));
	return retval;
}

int keyboard_getallbitdata(MZ2000 *mz2000)
{
	int i;
	int bitdata = 0xff;

	pthread_mutex_lock(&(mz2000->keyboard_mutex));
	for (i = 0; i < 0x0c; i++) bitdata &= mz2000->keyboard->key_bitdata[i];
	pthread_mutex_unlock(&(mz2000->keyboard_mutex));
	return bitdata;
}

int keyboard_breakkey_released(MZ2000 *mz2000)
{
	int retval;
	pthread_mutex_lock(&(mz2000->keyboard_mutex));
	retval = mz2000->keyboard->key_bitdata[3] & 0x80;
	pthread_mutex_unlock(&(mz2000->keyboard_mutex));
	return retval;
}

int keyboard_press(MZ2000 *mz2000, int keyval)
{
	KeyCode keycode;
	int mzkeycode;

	pthread_mutex_lock(&(mz2000->keyboard_mutex));
	keycode = XKeysymToKeycode(GDK_DISPLAY(), keyval);
	mzkeycode = mz2000->keyboard->kcode[keycode];
	if (!(mzkeycode == 0xff)) {
		int strobe = mzkeycode >> 3;
		int keybit = mzkeycode & 0x07;
		mz2000->keyboard->key_bitdata[strobe] &= ~(1 << keybit);
	}
	pthread_mutex_unlock(&(mz2000->keyboard_mutex));
	return 1;
}

int keyboard_release(MZ2000 *mz2000, int keyval)
{
	KeyCode keycode;
	int mzkeycode;

	pthread_mutex_lock(&(mz2000->keyboard_mutex));
	keycode = XKeysymToKeycode(GDK_DISPLAY(), keyval);
	mzkeycode = mz2000->keyboard->kcode[keycode];
	if (!(mzkeycode == 0xff)) {
		int strobe = mzkeycode >> 3;
		int keybit = mzkeycode & 0x07;
		mz2000->keyboard->key_bitdata[strobe] |= (1 << keybit);
	}
	pthread_mutex_unlock(&(mz2000->keyboard_mutex));
	return 1;
}

int keyboard_releaseall(MZ2000 *mz2000)
{
	int i;
	pthread_mutex_lock(&(mz2000->keyboard_mutex));
	for (i = 0; i < 16; i++) {
		mz2000->keyboard->key_bitdata[i] = 0xff;
	}
	pthread_mutex_unlock(&(mz2000->keyboard_mutex));
	return 1;
}
