#include <stdio.h>
#include <memory.h>
#include <pthread.h>
#include "mz2000.h"

static void loop_mz2000(MZ2000 *mz2000);
static int load_program(MZ2000 *mz2000, char *filename);

int main(int argc, char **argv)
{
	pthread_t mz2000_thread;
	pthread_t gtk_thread;
	MZ2000 *mz2000;
	int speed_khz = 0;

	g_thread_init(NULL);
	gtk_init(&argc, &argv);
	mz2000 = init_mz2000(FONTFILE, speed_khz);
	pthread_create(&gtk_thread, NULL, (void *)gtk_main, (void *)0);
	if (argc >= 2) {
		load_program(mz2000, argv[1]);
		mz2000->howipl = 0; /* no need */
	} else {
		mz2000->howipl = 1; /* internal function */
	}
	pthread_create(&mz2000_thread, NULL, (void *)loop_mz2000, (void *)mz2000);
	pthread_join(gtk_thread, NULL);
	return 0;
}

MZ2000 *init_mz2000(char *fontfile, int speed_khz)
{
	MZ2000 *ret;

	if ((ret = (MZ2000*)calloc(sizeof(MZ2000),1)) == NULL) {
		fprintf(stderr, "No enough memory\n");
		exit(1);
	}

	init_z80(speed_khz);
	init_io(ret);
	init_memory(ret);
	init_screen(ret, fontfile);
	init_keyboard(ret);
	init_tdeck(ret);
	return ret;
}

static void loop_mz2000(MZ2000 *mz2000)
{
	while(1) {
		switch(mz2000->howipl) {
		case 0: /* no need */
			memory_normal(mz2000);
			mz2000->howipl = 1;
			break;
		case 1: /* internal function */
			/* please write a function of ipl loader */
			memory_normal(mz2000);
			break;
		case 2: /* z80 program */
			memory_ipl(mz2000);
			break;
		}
		run_z80();
	}
}

int reset_mz2000(MZ2000 *mz2000)
{
	return reset_z80();
}

int iplreset_mz2000(MZ2000 *mz2000)
{
	return stop_z80();
}

static int load_program(MZ2000 *mz2000, char *filename)
{
	TAPE *intp, *outtp;

	if (filename == NULL) return 0;
	if ((intp = tropen(filename)) == NULL) return 0;
	if ((outtp = twopen_mem(mz2000->memory, 65536, TAPE_DATA2000B)) == NULL) return 0;

	tcopy(intp, outtp, 400);
	if (tclose(intp) < 0) return 0;
	if (tclose(outtp) < 0) return 0;

	return 1;
}
