#ifndef _MZ2000_H
#define _MZ2000_H

#include "z80.h"
#include "tape.h"
#include <gtk/gtk.h>
#include <X11/Xlib.h>

#define FONTFILE "font.pat"
#define FILENAMELEN 256

#define REFRESH_RATE 16

typedef struct mzfont {
	unsigned char chr40[256][128];
	unsigned char chr80[256][64];
} MZFont;

struct screen_region {
	int x1;
	int y1;
	int x2;
	int y2;
	int valid;
};

typedef struct mzscreen {
	GtkWidget *screen_widget;
	u8 scrbuf[640*400];
	u8 F4;
	u8 F5;
	u8 F6;
	char isgreen;
	char issize40;
	MZFont *font;
	guint32 pixel[8];
	guint32 colors[16];
	GdkRgbCmap *cmap;
	GdkGC *gc;
	pthread_mutex_t reg_mutex;
	struct screen_region region;
} MZScreen;

typedef struct mzkeyboard {
	KeyCode kcode[256];
	u8 key_bitdata[16];
} MZKeyboard;

typedef struct mzioinfo {
	u8 port_e8;
	int strobe_low;
} MZIOInfo;

#define TDECK_MAX_SLOTS 10

struct mztdeck_slotinfo {
	char *filename;
	int format;
};

typedef struct mztdeck {
	struct mztdeck_slotinfo slots[TDECK_MAX_SLOTS];
	int current;
	TAPE *tp;
	int used;
	int status;
} MZTDeck;

typedef struct mz2000 {
	u8 *ipl;
	u8 *memory;
	u8 *vram;
	u8 *gram1, *gram2, *gram3;
	int inipl, vramon, selchr, gramno;
	int howipl;
	/* 0: no need, 1: internal function, 2: z80 program */
	MZScreen *screen;
	MZKeyboard *keyboard;
	pthread_mutex_t keyboard_mutex;
	MZIOInfo *ioinfo;
	MZTDeck *tdeck;
	pthread_mutex_t tdeck_mutex;
#if 0
	GtkStatusbar *statusbar;
	guint statusbarid;
#endif
} MZ2000;

#if 0
#define TAPE_SELECTING 0
#define TAPE_FAILURE 1
#define TAPE_SUCCESS 2
#endif

/* mz2000.c */
MZ2000 *init_mz2000();
int reset_mz2000();
int iplreset_mz2000();

/* m2io.c */
int init_io(MZ2000 *mz);

/* m2mem.c */
int init_memory(MZ2000 *mz);
int memory_ipl(MZ2000 *mz);
int memory_normal(MZ2000 *mz);
int vram_on(MZ2000 *mz);
int vram_off(MZ2000 *mz);
int gram_on(MZ2000 *mz);
int gram_off(MZ2000 *mz);

/* screen.c */
int init_screen(MZ2000 *mz2000, char *fontfile);
int screen_portF4(MZ2000 *mz2000, unsigned char val);
int screen_portF5(MZ2000 *mz2000, unsigned char val);
int screen_portF6(MZ2000 *mz2000, unsigned char val);
int screen_color(MZ2000 *mz2000);
int screen_green(MZ2000 *mz2000);
int screen_size40(MZ2000 *mz2000);
int screen_size80(MZ2000 *mz2000);
int screen_redraw_all(MZ2000 *mz2000);
int screen_redraw_region(MZ2000 *mz2000);
int screen_redraw(MZ2000 *mz2000, int x, int y, int width, int height);
int screen_region_add_area(MZ2000 *mz2000, int x, int y, int width, int height);

/* mzgtk.c */
void mzgtk_error(char *message, GtkWidget *parent);
gint mzgtk_warn(char *message, GtkWidget *parent);
void mzgtk_file_select_read(MZ2000 *mz2000);
void mzgtk_file_select_write(MZ2000 *mz2000);
void mzgtk_init_window(MZ2000 *mz2000);

/* keyboard.c */
int init_keyboard(MZ2000 *mz2000);
int keyboard_getbitdata(MZ2000 *mz2000, int strobe);
int keyboard_getallbitdata(MZ2000 *mz2000);
int keyboard_breakkey_released(MZ2000 *mz2000);
int keyboard_press(MZ2000 *mz2000, int keyval);
int keyboard_release(MZ2000 *mz2000, int keyval);
int keyboard_releaseall(MZ2000 *mz2000);

/* tdeck.c */
int init_tdeck(MZ2000 *mz2000);
int tdeck_status(MZ2000 *mz2000);
int tdeck_slotused(MZ2000 *mz2000);
int tdeck_stop(MZ2000 *mz2000);
int tdeck_eject(MZ2000 *mz2000);
int tdeck_set(MZ2000 *mz2000);
int tdeck_ff(MZ2000 *mz2000);
int tdeck_rew(MZ2000 *mz2000);
int tdeck_clear(MZ2000 *mz2000);
int tdeck_insert(MZ2000 *mz2000, char *filename, int format, int slot);
int tdeck_delete(MZ2000 *mz2000, int slot);
int tdeck_add(MZ2000 *mz2000, char *filename, int format);
int tdeck_ropen(MZ2000 *mz2000);
int tdeck_wopen(MZ2000 *mz2000);
int tdeck_skip(MZ2000 *mz2000, z80clk clock);
int tdeck_scan(MZ2000 *mz2000);
int tdeck_write(MZ2000 *mz2000, int bit, z80clk clock);

#define TDECK_EJECTED 1
#define TDECK_STOPPED 2
#define TDECK_READING 3
#define TDECK_WRITING 4

#endif /* _MZ2000_H */
