#include <stdio.h>
#include <memory.h>
#include "mz2000.h"

static MZ2000 *mz2000_for_io;

static u8 in_e0(u16 port), in_e1(u16 port);
static u8 in_e2(u16 port), in_e3(u16 port);
static u8 in_e4(u16 port), in_e5(u16 port);
static u8 in_e6(u16 port), in_e7(u16 port);
static u8 in_e8(u16 port), in_e9(u16 port);
static u8 in_ea(u16 port), in_eb(u16 port);
static u8 in_f4(u16 port), in_f5(u16 port);
static u8 in_f6(u16 port), in_f7(u16 port);

static int out_e0(u16 port, u8 value), out_e1(u16 port, u8 value);
static int out_e2(u16 port, u8 value), out_e3(u16 port, u8 value);
static int out_e4(u16 port, u8 value), out_e5(u16 port, u8 value);
static int out_e6(u16 port, u8 value), out_e7(u16 port, u8 value);
static int out_e8(u16 port, u8 value), out_e9(u16 port, u8 value);
static int out_ea(u16 port, u8 value), out_eb(u16 port, u8 value);
static int out_f4(u16 port, u8 value), out_f5(u16 port, u8 value);
static int out_f6(u16 port, u8 value), out_f7(u16 port, u8 value);

int init_io(MZ2000 *arg)
{
	mz2000_for_io = arg;

	if ((arg->ioinfo = malloc(sizeof(MZIOInfo))) == NULL) {
		fprintf(stderr, "No enough memory\n");
		return 0;
	}

	set_ioent(0xe0, &in_e0, &out_e0);
	set_ioent(0xe1, &in_e1, &out_e1);
	set_ioent(0xe2, &in_e2, &out_e2);
	set_ioent(0xe3, &in_e3, &out_e3);
	set_ioent(0xe4, &in_e4, &out_e4);
	set_ioent(0xe5, &in_e5, &out_e5);
	set_ioent(0xe6, &in_e6, &out_e6);
	set_ioent(0xe7, &in_e7, &out_e7);
	set_ioent(0xe8, &in_e8, &out_e8);
	set_ioent(0xe9, &in_e9, &out_e9);
	set_ioent(0xea, &in_ea, &out_ea);
	set_ioent(0xeb, &in_eb, &out_eb);
	set_ioent(0xf4, &in_f4, &out_f4);
	set_ioent(0xf5, &in_f5, &out_f5);
	set_ioent(0xf6, &in_f6, &out_f6);
	set_ioent(0xf7, &in_f7, &out_f7);

	arg->ioinfo->port_e8 = 0;
	arg->ioinfo->strobe_low = 0;
	return 0;
}

static u8 in_e0(u16 port)
{
	return 0;
}

static int out_e0(u16 port, u8 value)
{
	return 0;
}

static u8 in_e1(u16 port)
{
	return 0;
}

static int out_e1(u16 port, u8 value)
{
	return 0;
}

static u8 in_e2(u16 port)
{
	return 0;
}

static int out_e2(u16 port, u8 value)
{
	return 0;
}

static u8 in_e3(u16 port)
{
	return 0;
}

static int out_e3(u16 port, u8 value)
{
	return 0;
}

static u8 in_e4(u16 port)
{
	return 0;
}

static int out_e4(u16 port, u8 value)
{
	return 0;
}

static u8 in_e5(u16 port)
{
	static int ret;
	ret = (ret == 0xc1)? 0xa8 : 0xc1;
	return ret;
}

static int out_e5(u16 port, u8 value)
{
	return 0;
}

static u8 in_e6(u16 port)
{
	return 0;
}

static int out_e6(u16 port, u8 value)
{
	return 0;
}

static u8 in_e7(u16 port)
{
	return 0;
}

static int out_e7(u16 port, u8 value)
{
	return 0;
}

static u8 in_e8(u16 port)
{
	return mz2000_for_io->ioinfo->port_e8;
}

static int out_e8(u16 port, u8 value)
{
	/* Bit 7,6 gram/vram switching */
	if ((value & 0xc0) != (mz2000_for_io->ioinfo->port_e8 & 0xc0)) {
		int oldvramon = mz2000_for_io->vramon;
		int newvramon = value & 0x80;
		int oldselchr = mz2000_for_io->selchr;
		int newselchr = value & 0x40;
		if (newvramon) {
			if (oldvramon) {
				if (newselchr) {
					gram_off(mz2000_for_io); vram_on(mz2000_for_io);
				} else {
					vram_off(mz2000_for_io); gram_on(mz2000_for_io);
				}
			} else {
				if (newselchr) vram_on(mz2000_for_io);
				else gram_on(mz2000_for_io);
			}
		} else {
			if (oldvramon) {
				if (oldselchr) vram_off(mz2000_for_io);
				else gram_off(mz2000_for_io);
			}
		}
	}

	/* Bit 5  width 80/40 */
	if ((value & 0x20) != (mz2000_for_io->ioinfo->port_e8 & 0x20)) {
		if (value & 0x20) screen_size80(mz2000_for_io);
		else screen_size40(mz2000_for_io);
	}

	/* Bit 4  key strobe low */
	mz2000_for_io->ioinfo->strobe_low = (!(value & 0x10));

	/* Bit 3,2,1,0  key strobe */

	mz2000_for_io->ioinfo->port_e8 = value;
	return 0;
}
static u8 in_e9(u16 port)
{
	return 0;
}

static int out_e9(u16 port, u8 value)
{
	return 0;
}

static u8 in_ea(u16 port)
{
	u8 code;
	if (mz2000_for_io->ioinfo->strobe_low) {
		code = keyboard_getallbitdata(mz2000_for_io);
	} else {
		code = keyboard_getbitdata(mz2000_for_io, mz2000_for_io->ioinfo->port_e8 & 0x0f);
	}
	return code;
}

static int out_ea(u16 port, u8 value)
{
	fprintf(stderr, "Warning: port EA is for input\n");
	return 0;
}

static u8 in_eb(u16 port)
{
	return 0;
}

static int out_eb(u16 port, u8 value)
{
	return 0;
}

static u8 in_f4(u16 port)
{
	fprintf(stderr, "Warning: port F4 is for output\n");
	return 0;
}

static int out_f4(u16 port, u8 value)
{
	screen_portF4(mz2000_for_io, value);
	return 0;
}

static u8 in_f5(u16 port)
{
	fprintf(stderr, "Warning: port F5 is for output\n");
	return 0;
}

static int out_f5(u16 port, u8 value)
{
	screen_portF5(mz2000_for_io, value);
	return 0;
}

static u8 in_f6(u16 port)
{
	fprintf(stderr, "Warning: port F6 is for output\n");
	return 0;
}

static int out_f6(u16 port, u8 value)
{
	screen_portF6(mz2000_for_io, value);
	return 0;
}

static u8 in_f7(u16 port)
{
	fprintf(stderr, "Warning: port F7 is for output\n");
	return mz2000_for_io->gramno;
}

static int out_f7(u16 port, u8 value)
{
	if (mz2000_for_io->gramno != value) {
		mz2000_for_io->gramno = value;
		if (mz2000_for_io->vramon && !mz2000_for_io->selchr) {
			gram_on(mz2000_for_io);
		}
	}
	return 0;
}
