#include <stdio.h>
#include <memory.h>
#include "mz2000.h"

static MZ2000 *mz;

static u8 vram_read(u16 addr);
static u8 gram1_read(u16 addr);
static u8 gram2_read(u16 addr);
static u8 gram3_read(u16 addr);
static int vram_write(u16 addr, u8 value);
static int gram1_write(u16 addr, u8 value);
static int gram2_write(u16 addr, u8 value);
static int gram3_write(u16 addr, u8 value);

int init_memory(MZ2000 *arg)
{
    mz = arg;
    if ((mz->ipl = (u8 *)malloc(2048)) == NULL) {
	fprintf(stderr, "No enough memory (memory)\n");
	exit(1);
    }
    if ((mz->memory = (u8 *)malloc(65536)) == NULL) {
	fprintf(stderr, "No enough memory (memory)\n");
	exit(1);
    }
    if ((mz->vram = (u8 *)malloc(2048)) == NULL) {
	fprintf(stderr, "No enough memory (vram)\n");
	exit(1);
    }
    if ((mz->gram1 = (u8 *)malloc(16384)) == NULL) {
	fprintf(stderr, "No enough memory (gram1)\n");
	exit(1);
    }
    if ((mz->gram2 = (u8 *)malloc(16384)) == NULL) {
	fprintf(stderr, "No enough memory (gram2)\n");
	exit(1);
    }
    if ((mz->gram3 = (u8 *)malloc(16384)) == NULL) {
	fprintf(stderr, "No enough memory (gram3)\n");
	exit(1);
    }

    mz->selchr = 1;
    mz->gramno = 0;

    memory_ipl(mz);
    return 1;
}

static u8 vram_read(u16 addr)
{
    addr &= 0xfff;
    return (addr < 0x800)? mz->vram[addr] : 0;
}

static u8 gram1_read(u16 addr)
{
    addr &= 0x3fff;
    return mz->gram1[addr];
}

static u8 gram2_read(u16 addr)
{
    addr &= 0x3fff;
    return mz->gram2[addr];
}

static u8 gram3_read(u16 addr)
{
    addr &= 0x3fff;
    return mz->gram3[addr];
}

static int vram_write(u16 addr, u8 value)
{
    addr &= 0xfff;
    if (addr < 0x800) {
	mz->vram[addr] = value;
    }
    if (mz->screen->issize40) {
	if (addr < 40 * 25) {
	    screen_region_add_area(mz, (addr % 40)*16, addr/40*16, 16, 16);
	}
    } else {
	if (addr < 80 * 25) {
	    screen_region_add_area(mz, (addr % 80)*8, addr/80*16, 8, 16);
	}
    }
    return 1;
}

static int gram1_write(u16 addr, u8 value)
{
    addr &= 0x3fff;
    mz->gram1[addr] = value;
    if (addr < 80 * 200) {
	screen_region_add_area(mz, (addr % 80)*8, addr/80*2, 8, 2);
    }
    return 1;
}

static int gram2_write(u16 addr, u8 value)
{
    addr &= 0x3fff;
    mz->gram2[addr] = value;
    if (addr < 80 * 200) {
	screen_region_add_area(mz, (addr % 80)*8, addr/80*2, 8, 2);
    }
    return 1;
}

static int gram3_write(u16 addr, u8 value)
{
    addr &= 0x3fff;
    mz->gram3[addr] = value;
    if (addr < 80 * 200) {
	screen_region_add_area(mz, (addr % 80)*8, addr/80*2, 8, 2);
    }
    return 1;
}

int memory_ipl(MZ2000 *mz)
{
    int i;

    set_mement(0, mz->ipl, NULL, NULL);
    for ( i = 1; i < 8; i++ ) set_mement(i, NULL, NULL, NULL);
    for ( i = 0; i < 8; i++ ) set_mement(i+8, mz->memory+i*4096, NULL, NULL);

    mz->inipl = 1;
    mz->vramon = 0;

    return 1;
}

int memory_normal(MZ2000 *mz)
{
    int i;
    for ( i = 0; i < 16; i++ ) set_mement(i, mz->memory+i*4096, NULL, NULL);
    mz->inipl = 0;
    mz->vramon = 0;

    return 1;
}

int vram_on(MZ2000 *mz)
{
    set_mement(13, NULL, &vram_read, &vram_write);
    mz->vramon = 1;
    mz->selchr = 1;
    return 1;
}

int gram_on(MZ2000 *mz)
{
    u8 (*readf)(u16);
    int (*writef)(u16 addr,u8 value);

    switch (mz->gramno) {
    case 1:
	readf = &gram1_read;
	writef = &gram1_write;
	break;
    case 2:
	readf = &gram2_read;
	writef = &gram2_write;
	break;
    case 3:
	readf = &gram3_read;
	writef = &gram3_write;
	break;
    default:
	readf = NULL;
	writef = NULL;
	break;
    }
    set_mement(12, NULL, readf, writef);
    set_mement(13, NULL, readf, writef);
    set_mement(14, NULL, readf, writef);
    set_mement(15, NULL, readf, writef);

    mz->vramon = 1;
    mz->selchr = 0;
    return 1;
}

int vram_off(MZ2000 *mz)
{
    if (mz->inipl) {
	set_mement(13, mz->memory+4096*5, NULL, NULL);
    } else {
	set_mement(13, mz->memory+4096*13, NULL, NULL);
    }

    mz->vramon = 0;
    return 1;
}

int gram_off(MZ2000 *mz)
{
    if (mz->inipl) {
	set_mement(12, mz->memory+4096*4, NULL, NULL);
	set_mement(13, mz->memory+4096*5, NULL, NULL);
	set_mement(14, mz->memory+4096*6, NULL, NULL);
	set_mement(15, mz->memory+4096*7, NULL, NULL);
    } else {
	set_mement(12, mz->memory+4096*12, NULL, NULL);
	set_mement(13, mz->memory+4096*13, NULL, NULL);
	set_mement(14, mz->memory+4096*14, NULL, NULL);
	set_mement(15, mz->memory+4096*15, NULL, NULL);
    }

    mz->vramon = 0;
    return 1;
}

