#include <sys/stat.h>
#include <errno.h>
#include "mz2000.h"

/*
 *   mzgtk_error
 *   mzgtk_warn
 *      error/warning window
 */
static int warn_ok(GtkWidget *widget, gint *ret)
{
    *ret = TRUE;
    return FALSE;
}

static int warn_cancel(GtkWidget *widget, gint *ret)
{
    *ret = FALSE;
    return FALSE;
}

static gint mzgtk_errwarn(char *message, gint iswarn, GtkWidget *parent)
{
    GtkWidget *window, *label, *button1, *button2;
    gint ret;

    window = gtk_dialog_new();

    gtk_signal_connect(GTK_OBJECT (window), "destroy",
		       GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
    gtk_window_set_title(GTK_WINDOW(window), (iswarn)? "Warning!": "Error!");
    gtk_container_set_border_width(GTK_CONTAINER(window), 0);
    gtk_widget_set_usize(window, 200, 100);

    button1 = gtk_button_new_with_label("OK");
    gtk_signal_connect(GTK_OBJECT(button1), "clicked",
		       GTK_SIGNAL_FUNC(warn_ok), &ret);
    gtk_signal_connect_object(GTK_OBJECT(button1), "clicked",
			      GTK_SIGNAL_FUNC(gtk_widget_destroy),
			      GTK_OBJECT(window));
    GTK_WIDGET_SET_FLAGS(button1, GTK_CAN_DEFAULT);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area), button1,
		       TRUE, TRUE, 0);
    gtk_widget_grab_default(button1);
    gtk_widget_show(button1);

    if (iswarn) {
	button2 = gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(button2), "clicked",
			   GTK_SIGNAL_FUNC(warn_cancel), &ret);
	gtk_signal_connect_object(GTK_OBJECT(button2), "clicked",
				  GTK_SIGNAL_FUNC(gtk_widget_destroy),
				  GTK_OBJECT(window));
	GTK_WIDGET_SET_FLAGS(button2, GTK_CAN_DEFAULT);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area), button2,
			   TRUE, TRUE, 0);
	gtk_widget_show(button2);
    }

    label = gtk_label_new(message);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), label,
		       TRUE, TRUE, 0);
    gtk_widget_show(label);

    gtk_window_set_modal(GTK_WINDOW(window), TRUE);
    if (parent != NULL) {
	gint x, y;
	gtk_window_set_transient_for(GTK_WINDOW(window), GTK_WINDOW(parent));
	gdk_window_get_root_origin(parent->window, &x, &y);
	gtk_widget_set_uposition(window, x+40, y+40);
    }

    gtk_widget_show(window);
    gtk_main();
    return ret;
}

void mzgtk_error(char *message, GtkWidget *parent) 
{
    mzgtk_errwarn(message, FALSE, parent);
}

gint mzgtk_warn(char *message, GtkWidget *parent) 
{
    return mzgtk_errwarn(message, TRUE, parent);
}

#if 0
/*
 *   mzgtk_select_file_read
 *   mzgtk_select_file_write
 *      selecting files for reading/writing
 */
static int tape_format;
static TAPE *tape;

static void fileread_ok(GtkWidget *w, GtkFileSelection *fs)
{
    gchar *filename;
    struct stat buf;
    int v;

    filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
    v = stat(filename, &buf);
    if (v == -1) {
	mzgtk_error("Can't file open\n (Maybe no files existing)",
		    GTK_WIDGET(fs));
	return;
    }

    tape = tropen(filename);
    if (tape == NULL) {
	mzgtk_error("Not valid format\n", GTK_WIDGET(fs));
    } else {
	printf("%s\n", filename);
	gtk_widget_destroy(GTK_WIDGET(fs));
    }
}

static void filewrite_ok(GtkWidget *w, GtkFileSelection *fs)
{
    gchar *filename;
    struct stat buf;
    int v;

    filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));

    v = stat(filename, &buf);
    if ((v == -1)&&(errno = ENOENT)) {
	tape = twopen(filename, tape_format);
	if (tape == NULL) {
	    mzgtk_error("Can't open file\n", GTK_WIDGET(fs));
	} else {
	    printf("%s\n", filename);
	    gtk_widget_destroy(GTK_WIDGET(fs));
	}
    } else {
	mzgtk_error("Can't open file\n", GTK_WIDGET(fs));
    }
}

static void file_cancel(GtkWidget *w, GtkFileSelection *fs)
{
    tape = NULL;
    gtk_widget_destroy(GTK_WIDGET(fs));
}

static void tape_format_rlf(GtkWidget *w, MZ2000 *mz2000)
{
    tape_format = TAPE_RUNLENGTH;
}

static void tape_format_mzf(GtkWidget *w, MZ2000 *mz2000)
{
    tape_format = TAPE_FILE2000B;
}

static void mzgtk_select_file(MZ2000 *mz2000, int forread)
{
    GtkWidget *filew;

    if (forread) {
	filew = gtk_file_selection_new ("Tape Selection for Reading");
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->ok_button),
			   "clicked", (GtkSignalFunc) fileread_ok,
			   GTK_OBJECT(filew));
    } else {
	filew = gtk_file_selection_new ("Tape Selection for Writing");
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->ok_button),
			   "clicked", (GtkSignalFunc) filewrite_ok,
			   GTK_OBJECT(filew));
    }

    gtk_signal_connect(GTK_OBJECT(filew), "destroy",
		       GTK_SIGNAL_FUNC(gtk_main_quit), NULL);

    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->cancel_button),
		       "clicked", (GtkSignalFunc) file_cancel,
		       GTK_OBJECT(filew));

    if (!forread) {
	GtkWidget *button1, *button2;
	GSList *glist;

	button1 = gtk_radio_button_new_with_label(NULL, "mzf format");
	glist = gtk_radio_button_group(GTK_RADIO_BUTTON(button1));
	button2 = gtk_radio_button_new_with_label(glist, "rlf format");
	glist = gtk_radio_button_group(GTK_RADIO_BUTTON(button2));

	gtk_signal_connect(GTK_OBJECT (button1), "clicked",
			   GTK_SIGNAL_FUNC(tape_format_mzf),
			   mz2000);
	gtk_signal_connect(GTK_OBJECT (button2), "clicked",
			   GTK_SIGNAL_FUNC(tape_format_rlf),
			   mz2000);

	gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(filew)->main_vbox),
			   button1, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(filew)->main_vbox),
			   button2, TRUE, TRUE, 0);
	gtk_widget_show(button1);
	gtk_widget_show(button2);
	tape_format = TAPE_FILE2000B;
    }
    gtk_widget_show(filew);
    mz2000->tape_select_status = TAPE_SELECTING;
    gtk_main();

    if (tape != NULL) {
	mz2000->seltape = tape;
	mz2000->tape_select_status = TAPE_SUCCESS;
    } else {
	mz2000->seltape = NULL;
	mz2000->tape_select_status = TAPE_FAILURE;
    }
}

void mzgtk_file_select_read(MZ2000 *mz2000)
{
    mzgtk_select_file(mz2000, TRUE);
}

void mzgtk_file_select_write(MZ2000 *mz2000)
{
    mzgtk_select_file(mz2000, FALSE);
}
#endif

/*
 *   mzgtk_mainwindow
 *      create a main window
 */

static GtkWidget *main_window_for_item;
static MZ2000 *mz2000_for_item;

static void mzgtk_quit()
{
    gint warn;
    warn = mzgtk_warn("Quit OK?", main_window_for_item);
    if (warn) {
	gtk_main_quit();
    }
}

static void mzgtk_green()
{
    screen_green(mz2000_for_item);
}

static void mzgtk_color()
{
    screen_color(mz2000_for_item);
}

static void mzgtk_setspeed(gpointer func_data, guint speed,
			   GtkWidget *widget)
{
    setspeed_z80(speed);
}

static void mzgtk_reset()
{
	reset_mz2000(mz2000_for_item);
}

static void mzgtk_iplreset()
{
	iplreset_mz2000(mz2000_for_item);
}

static GtkWidget *mzgtk_menubar(GtkWidget *main_window, MZ2000 *mz2000)
{
    GtkWidget *menubar;
    GtkItemFactory *item_factory;
    GtkAccelGroup *accel_group;

    GtkItemFactoryEntry menu_items[] = {
	{"/File", NULL, NULL, 0, "<Branch>" },
	{"/File/Quit", "<Control>Q", mzgtk_quit, 0, NULL},
	{"/Display", NULL, NULL, 0, "<Branch>" },
	{"/Display/Green", "<Control>G", mzgtk_green, 0, "<RadioItem>"},
	{"/Display/Color", "<Control>C", mzgtk_color, 0, "/Display/Green"},
	{"/Button", NULL, NULL, 0, "<Branch>"},
	{"/Button/Reset", "<Control>R", (GtkItemFactoryCallback)mzgtk_reset, 0, NULL},
	{"/Button/IPLReset", "<Control>I", (GtkItemFactoryCallback)mzgtk_iplreset, 0, NULL},
	{"/Speed", NULL, NULL, 0, "<Branch>"},
	{"/Speed/No wait", NULL, (GtkItemFactoryCallback)mzgtk_setspeed, 0, "<RadioItem>"},
	{"/Speed/4MHz", NULL, (GtkItemFactoryCallback)mzgtk_setspeed, 4000, "/Speed/No wait"},
	{"/Debug", NULL, NULL, 0, "<LastBranch>"},
	{"/Debug/PrintReg", "<Control>P", print_registers, 0, NULL},
#ifdef TRACE
	{"/Debug/TraceOff", NULL, trace_off, 0, "<RadioItem>"},
	{"/Debug/TraceOn", NULL, trace_on, 0, "/Debug/TraceOff"},
#endif
    };

    gint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);

    accel_group = gtk_accel_group_new();
    item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>",
					accel_group);
    gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);
    gtk_accel_group_attach(accel_group, GTK_OBJECT(main_window));
    menubar = gtk_item_factory_get_widget(item_factory, "<main>");
    mz2000_for_item = mz2000;
    main_window_for_item = main_window;
    return menubar;
}

static gint mzgtk_screen_expose(GtkWidget *main_window, GdkEventExpose *event,
				MZ2000 *mz2000)
{
    screen_redraw(mz2000, event->area.x, event->area.y, event->area.width,
		  event->area.height);
    return FALSE;
}

static gint mzgtk_key_press(GtkWidget *main_window, GdkEventKey *event,
			    MZ2000 *mz2000)
{
    keyboard_press(mz2000, event->keyval);
    return FALSE;
}

static gint mzgtk_key_release(GtkWidget *main_window, GdkEventKey *event,
			      MZ2000 *mz2000)
{
    keyboard_release(mz2000, event->keyval);
    return FALSE;
}

static GtkWidget *mzgtk_screen(GtkWidget *main_window, MZ2000 *mz2000)
{
    GtkWidget *screen;

    screen = gtk_drawing_area_new();
    gtk_drawing_area_size(GTK_DRAWING_AREA(screen), 640, 400);
    gtk_signal_connect_object(GTK_OBJECT(screen), "destroy",
			      GTK_SIGNAL_FUNC(gtk_widget_destroy),
			      GTK_OBJECT(main_window));
    gtk_widget_add_events(screen,
			    GDK_EXPOSURE_MASK
			  | GDK_ENTER_NOTIFY_MASK
			  | GDK_LEAVE_NOTIFY_MASK
			  | GDK_KEY_PRESS_MASK
			  | GDK_KEY_RELEASE_MASK);
    gtk_signal_connect(GTK_OBJECT(screen), "expose_event",
		       GTK_SIGNAL_FUNC(mzgtk_screen_expose),
		       mz2000);
    gtk_signal_connect_object(GTK_OBJECT(screen), "enter_notify_event",
			      GTK_SIGNAL_FUNC(gdk_key_repeat_disable), NULL);
    gtk_signal_connect_object(GTK_OBJECT(screen), "leave_notify_event",
			      GTK_SIGNAL_FUNC(gdk_key_repeat_restore), NULL);
    gtk_signal_connect(GTK_OBJECT(main_window), "key_press_event",
		       GTK_SIGNAL_FUNC(mzgtk_key_press),
		       mz2000);
    gtk_signal_connect(GTK_OBJECT(main_window), "key_release_event",
		       GTK_SIGNAL_FUNC(mzgtk_key_release),
		       mz2000);
    return screen;
}

static int mzgtk_refresh_screen(MZ2000 *mz2000)
{
    screen_redraw_region(mz2000);
    return TRUE;
}

void mzgtk_init_window(MZ2000 *mz2000)
{
    GtkWidget *main_window, *vbox, *menubar, *screen;

    main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_widget_set_name(main_window, "MZ-2000 Screen");
    gtk_signal_connect(GTK_OBJECT(main_window), "destroy",
		       GTK_SIGNAL_FUNC(gtk_main_quit), NULL);

    vbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(main_window), vbox);
    gtk_signal_connect_object(GTK_OBJECT(vbox), "destroy",
			      GTK_SIGNAL_FUNC(gtk_widget_destroy),
			      GTK_OBJECT(main_window));
    gtk_widget_show(vbox);

    menubar = mzgtk_menubar(main_window, mz2000);
    gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, TRUE, 0);
    gtk_widget_show(menubar);

    screen = mzgtk_screen(main_window, mz2000);
    gtk_box_pack_start(GTK_BOX(vbox), screen, TRUE, TRUE, 0);
    gtk_widget_show(screen);
    mz2000->screen->screen_widget = screen;

    gtk_widget_show(main_window);

    gtk_timeout_add(REFRESH_RATE, (GtkFunction) mzgtk_refresh_screen, mz2000);
}

#if 0
int main(int argc, char **argv)
{
    MZ2000 *mz2000;
    int i;
    gint ret;

    gtk_init(&argc, &argv);


    mz2000 = init_mz2000("font.pat");
#if 0
    mzgtk_file_select_read(mz2000);
#endif

    for (i = 0; i < 256; i++) {
	mz2000->vram[i] = i;
    }
    screen_ram2buf_all(mz2000);
    gtk_main();

    return 0;
}
#endif
