#include "z80.h"
#include "z80operand.h"
#include <glib.h>
#include <stdlib.h>
#include <stdio.h>

Z80 z80;
char is_peven[256];
int daa_array[2048];

static int z80timer();
static int main_z80();

int init_z80(int speed_khz)
{
	int i;

	pthread_mutex_init(&(z80.z80_mutex), NULL);

	for (i = 0; i < 16; i++) {
		z80.mem[i].ptr = NULL;
		z80.mem[i].readf = NULL;
		z80.mem[i].writef = NULL;
	}
	for (i = 0; i < 256; i++) {
		z80.io[i].readf = NULL;
		z80.io[i].writef = NULL;
	}
#ifdef IO16BIT
	for (i = 256; i < 65536; i++) {
		z80.io[i].readf = NULL;
		z80.io[i].writef = NULL;
	}
#endif
	z80.a = z80.b = z80.c = z80.d = z80.e = z80.f = z80.h = z80.l = 0;
	z80.a2 = z80.b2 = z80.c2 = z80.d2 = z80.e2 = z80.f2 = z80.h2 = z80.l2 = 0;
	z80.ix = z80.iy = z80.sp = 0;
	z80.internal_clock = 0;
	z80.left_clock = 0;
	setspeed_z80(speed_khz);
	z80.onbus = 0;
#ifdef TRACE
	z80.trace = 0;
#endif

	for (i = 0; i < 256; i++) {
		int j,n = 0;
		for (j = 0; j < 8; j++) {
			if (((i >> j) & 1) == 1) n++;
		}
		is_peven[i] = ((n == 0)||(n == 2)||(n == 4)||(n == 6)||(n == 8));;
	}

	for (i = 0; i < 2048; i++) {
		int v = i & 0xff;
		int ih = (i >> 4) & 0x0f;
		int il = i & 0x0f;
		switch (i>>8) {
		case 0:
			if ((ih <= 8)&&(il >= 10)) v += 0x06;
			else if ((ih >= 10)&&(il <= 9)) v += 0x60;
			else if ((ih >= 9)&&(il >= 10)) v += 0x66;
			break;
		case 1:
			if ((ih <= 9)&&(il <= 3)) v += 0x06;
			else if ((ih >= 10)&&(il <= 3)) v += 0x66;
			break;
		case 2:
			if ((ih <= 2)&&(il <= 9)) v += 0x60 + 0x100;
			else if ((ih <= 2)&&(il >= 10)) v += 0x66 + 0x100;
			break;
		case 3:
			if ((ih <= 3)&&(il >= 3)) v += 0x66 + 0x100;
			break;
		case 5:
			if ((ih <= 8)&&(il >= 6)) v -= 0x06;
			break;
		case 6:
			if ((ih >= 7)&&(il <= 9)) v += 0xa0;
			break;
		case 7:
			if ((ih >= 6)&&(il >= 6)) v += 0x9a;
			break;
		}
		daa_array[v] = v;
	}
	z80.instructs[GROUP_XX] = instr_xx;
	z80.instructs[GROUP_CB] = instr_cb;
	z80.instructs[GROUP_ED] = instr_ed;
	z80.instructs[GROUP_DD] = instr_dd;
	z80.instructs[GROUP_FD] = instr_fd;
	z80.instructs[GROUP_DDCB] = instr_ddcb;
	z80.instructs[GROUP_FDCB] = instr_fdcb;

	z80.timer_wait = 0;
	pthread_mutex_init(&(z80.timer_mutex), NULL);
	pthread_cond_init(&(z80.timer_cond), NULL);
	g_timeout_add_full(0, Z80_TIMER_MSEC, z80timer, NULL, NULL);
	return 1;
}

static int z80timer()
{
	pthread_mutex_lock(&(z80.timer_mutex));
	if (z80.timer_wait) {
		z80.timer_wait = 0;
		pthread_cond_signal(&(z80.timer_cond));
	}
	pthread_mutex_unlock(&(z80.timer_mutex));
	return 1;
}

int run_z80()
{
	reset_z80();
	pthread_mutex_lock(&(z80.z80_mutex));
	z80.stop = 0;
	pthread_mutex_unlock(&(z80.z80_mutex));
	while (1) {
		if (z80.speed_khz == 0) {
			z80.left_clock += DEFAULT_SPEED_KHZ * Z80_TIMER_MSEC;
		} else {
			pthread_mutex_lock(&(z80.timer_mutex));
			z80.timer_wait = 1;
			pthread_cond_wait(&(z80.timer_cond), &(z80.timer_mutex));
			pthread_mutex_unlock(&(z80.timer_mutex));
			z80.left_clock += z80.speed_khz;
		}
		if (main_z80() == 0) break;
	}
	return 1;
}

static int main_z80()
{
	int clk;

	pthread_mutex_lock(&(z80.z80_mutex));
	if (z80.stop) {
		pthread_mutex_unlock(&(z80.z80_mutex));
		return 0;
	}
	if (z80.external_signal) {
		clk = exec_interrupt(z80.onbus);
		z80.internal_clock += clk;
		z80.left_clock -= clk;
	}
	while (z80.left_clock > 0) {
		clk = (z80.halt_state)? 4 : exec_instruction(GROUP_XX);
		z80.internal_clock += clk;
		z80.left_clock -= clk;
#ifdef TRACE
		if (z80.trace) {
			print_registers();
		}
#endif
	}
	pthread_mutex_unlock(&(z80.z80_mutex));
	return 1;
}

int reset_z80()
{
	pthread_mutex_lock(&(z80.z80_mutex));
	z80.external_signal |= RESET_SIGNAL;
	z80.onbus = 0;
	z80.halt_state = 0;
	pthread_mutex_unlock(&(z80.z80_mutex));
	return 1;
}

int int_z80(int onbus)
{
	pthread_mutex_lock(&(z80.z80_mutex));
	z80.external_signal |= INT_SIGNAL;
	z80.onbus = onbus;
	z80.halt_state = 0;
	pthread_mutex_unlock(&(z80.z80_mutex));
	return 1;
}

int nmi_z80()
{
	pthread_mutex_lock(&(z80.z80_mutex));
	z80.external_signal |= NMI_SIGNAL;
	z80.onbus = 0;
	z80.halt_state = 0;
	pthread_mutex_unlock(&(z80.z80_mutex));
	return 1;
}

int stop_z80()
{
	pthread_mutex_lock(&(z80.z80_mutex));
	z80.stop = 1;
	pthread_mutex_unlock(&(z80.z80_mutex));
	return 1;
}

z80clk get_internal_clock()
{
	return z80.internal_clock;
}

int setspeed_z80(int speed_khz)
{
	pthread_mutex_lock(&(z80.z80_mutex));
	z80.speed_khz = speed_khz * Z80_TIMER_MSEC;
	pthread_mutex_unlock(&(z80.z80_mutex));
	return 1;
}

#ifdef TRACE
void trace_on()
{
	z80.trace = 1;
}

void trace_off()
{
	z80.trace = 0;
}
#endif

void print_registers()
{
	printf("A B C D E H L F  IX  IY  SP  PC   Clock\n");
	printf("%02x%02x%02x%02x%02x%02x%02x%02x %04x%04x%04x%04x %qd\n", 
	       z80.a, z80.b, z80.c, z80.d, z80.e, z80.h, z80.l, z80.f,
	       z80.ix, z80.iy, z80.sp, z80.pc, z80.internal_clock);
	printf("%02x%02x%02x%02x%02x%02x%02x%02x\n", 
	       z80.a2, z80.b2, z80.c2, z80.d2, z80.e2, z80.h2, z80.l2, z80.f2);
	printf("Memory: %04x %02x%02x%02x%02x%02x ...\n",
	       z80.pc, memread8(z80.pc), memread8(z80.pc+1), memread8(z80.pc+2),
	       memread8(z80.pc+3), memread8(z80.pc+4));
	printf("Stack: %04x %04x %04x %04x %04x ...\n",
	       memread16(z80.sp), memread16(z80.sp+2), memread8(z80.sp+4),
	       memread8(z80.sp+6), memread8(z80.sp+8));
}

int set_mement(int segment, u8 *ptr,
	       u8 (*readf)(u16 addr),
	       int (*writef)(u16 addr,u8 value))
{
	segment &= 0xf;
	z80.mem[segment].ptr = ptr;
	z80.mem[segment].readf = readf;
	z80.mem[segment].writef = writef;
	return 1;
}

int set_ioent(int port,
	      u8 (*readf)(u16 port),
	      int (*writef)(u16 port, u8 value))
{
#ifndef IO16BIT
	port &= 0xff;
#endif
	z80.io[port].readf = readf;
	z80.io[port].writef = writef;
	return 1;
}

