#ifndef _Z80_H
#define _Z80_H

#include <pthread.h>

typedef unsigned char u8;
typedef signed char s8;
typedef unsigned short u16;
typedef signed short s16;
typedef long long z80clk;

struct z80mement {
	u8 *ptr;
	u8 (*readf)(u16 addr);
	int (*writef)(u16 addr, u8 value);
};

struct z80ioent {
	u8 (*readf)(u16 port);
	int (*writef)(u16 port, u8 value);
};

typedef struct z80 {
	struct z80mement mem[16];
#ifdef IO16BIT
	struct z80ioent io[65536];
#else
	struct z80ioent io[256];
#endif
	u8 a,b,c,d,e,f,h,l;
	u8 a2,b2,c2,d2,e2,f2,h2,l2;
	u16 ix,iy,sp,pc;
	u8 r,i,iff1,iff2,im;
	z80clk internal_clock;
	z80clk left_clock;
	int speed_khz;
	int halt_state;
	int timer_wait;
	pthread_cond_t timer_cond;
	pthread_mutex_t timer_mutex;
	int external_signal;
	int onbus;
	int stop;
	pthread_mutex_t z80_mutex;
#ifdef TRACE
	int trace;
#endif
	int (*(*instructs[7]))();
} Z80;

extern Z80 z80;

#define NO_SIGNAL 0
#define RESET_SIGNAL 1
#define NMI_SIGNAL 2 
#define INT_SIGNAL 4

#define DEFAULT_SPEED_KHZ 4000
#define Z80_TIMER_MSEC 10

int init_z80(int speed_khz);
int set_mement(int segment, u8 *ptr,
	       u8 (*readf)(u16 addr),
	       int (*writef)(u16 addr,u8 value));
int set_ioent(int port,
	      u8 (*readf)(u16 port),
	      int (*writef)(u16 port, u8 value));
int start_z80();
int run_z80();
int reset_z80();
int int_z80(int onbus);
int nmi_z80();
int stop_z80();
z80clk get_internal_clock();
int setspeed_z80(int speed_khz);
#ifdef TRACE
void trace_on();
void trace_off();
#endif
void print_registers();

#endif /* _Z80_H */
